// design_pattern.hh: this file is part of the Vaucanson project.
//
// Vaucanson, a generic library for finite state machines.
// Copyright (C) 2001, 2002, 2003, 2004 The Vaucanson Group.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// The Vaucanson Group consists of the following contributors:
//    * Jacques Sakarovitch <sakarovitch@enst.fr>
//    * Sylvain Lombardy <lombardy@liafa.jussieu.fr>
//    * Thomas Claveirole <thomas.claveirole@lrde.epita.fr>
//    * Loic Fosse <loic.fosse@lrde.epita.fr>
//    * Thanh-Hoc Nguyen <nguyen@enst.fr>
//    * Raphael Poss <raphael.poss@lrde.epita.fr>
//    * Yann Regis-Gianas <yann.regis-gianas@lrde.epita.fr>
//    * Maxime Rey <maxime.rey@lrde.epita.fr>
//    * Sarah O'Connor <sarah.o-connor@lrde.epita.fr>
//    * Louis-Noel Pouchet <louis-noel.pouchet@lrde.epita.fr>
//
#ifndef VCSN_DESIGN_PATTERN_DESIGN_PATTERN_HH
# define VCSN_DESIGN_PATTERN_DESIGN_PATTERN_HH

/** @addtogroup design_pattern Fundamental C++: the Element design pattern.
 *
 * Core constructs for Vaucanson.
 *
 * This module contains definitions for the design pattern used in
 * Vaucanson, so called "Element". This design pattern is more deeply
 * described in the paper by Yann Rgis-Gianas and Raphal Poss:
 * <em>On orthogonal specialization in C++: Dealing with efficiency
 * and algebraic abstraction in Vaucanson</em>. In the proceedings of
 * the Parallel/High-performance Object-Oriented Scientific Computing
 * (POOSC'2003) Darmstadt, Germany July 2003.
 *
 * @see http://www.lrde.epita.fr/cgi-bin/twiki/view/Publications/200307-Poosc
*/

# include <vaucanson/design_pattern/element.hh>

#endif // ! VCSN_DESIGN_PATTERN_DESIGN_PATTERN_HH
