/*  Vaucanson, a generic library for finite state machines.
Copyright (C) 2001, 2002, 2004 Sakarovitch, Lombardy, Poss, Rey and Regis-Gianas.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <cassert>
#include <vaucanson/design_pattern/design_pattern.hh>
#include <algebra/series/krat/derivations/derivations_common.hh>

#include <vaucanson/algebra/implementation/free_monoid/predefs.hh>
#include <vaucanson/algebra/implementation/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/implementation/series/series.hh>
#include <vaucanson/algebra/implementation/series/krat.hh>

template <class Expr>
class Derivation
{
public:
  typedef Expr						krat_exp_t;
  typedef Expr						result_t;
  typedef typename Expr::monoid_elt_t::set_t::letter_t	letter_t;
  
  Derivation(const krat_exp_t& expr, const letter_t& l) :
    result (derivate(expr, l))
  {
  }
  
  bool is_defined()
  {
    return result.second;
  }
  
  const krat_exp_t& get_result()
  {
    assert(result.second);
    return result.first;
  }
  
protected:
  std::pair<krat_exp_t, bool> result;
};

#define TEST_STATIC_ARG1				\
	vcsn::Element<					\
	  vcsn::algebra::Series<			\
	    vcsn::NumericalSemiring,			\
	    vcsn::algebra::char_letter::Words>,		\
	  vcsn::rat::exp<				\
	    vcsn::algebra::char_letter::Word::value_t,	\
	    bool> >

#define TEST_STATIC_ARG2				\
	Derivation<TEST_STATIC_ARG1>

#define TEST_STATIC_ARGS				\
	TEST_STATIC_ARG1, TEST_STATIC_ARG2


#include <check/tester.hh>
#include <krat_exp_derivation_identity_test.hh>

  int main(int argc, char **argv)
  {
  using namespace vcsn;
  tests::verbose_level verbose = tests::low;
  srand(time(0));

  if (argc > 1)
    {
      if (argv[1] == std::string("--no-verbose"))
	verbose = tests::none;
      else if (argv[1] == std::string ("--high-verbose"))
	verbose = tests::high;
      else
	verbose = tests::low;
    };
    tests::Tester t(verbose);
    if (krat_exp_derivation_identity_test<TEST_STATIC_ARGS>(t))
      return EXIT_SUCCESS;
    else
      return EXIT_FAILURE;
  }

